/* Compile with: gcc -o RunImage roxaudiogen.c `sdl-config --cflags` `sdl-config --libs` */

#include <stdlib.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <string.h>
#include <limits.h>
#include <signal.h>
#include <SDL.h>
#include "SDL_gfxPrimitives.c"
#include "fonts/SDL_gfx_10x20_fnt.h"
#include "fonts/SDL_gfx_9x18_fnt.h"
#include "fonts/SDL_gfx_9x18B_fnt.h"
#include "fonts/SDL_gfx_8x13O_fnt.h"

#include "PLOTlib.c"

char dummy;

SDL_Event event;

pid_t sound_pid;

static pid_t chld_pid;

int reason,quit,mouse_x,mouse_y,mouse_button;

int mouse_x,mouse_y;
int mouse_hit;
int icount,tone_on,switch_on,istop;
char mstring[100];
char teststring[512];
char teststring2[512];
char commandstring[1024];
char appdirname[1024];
char charzero;

float f,a,r,f_playing,a_playing,r_playing;
float f_preset[5], a_preset[5], r_preset[5];
float rates_available[4];
int centi_a,i,pending,rind,bps;

struct valuebox {
 char name[32];
 char units[8];
 int value;
 int x0;
 int y0;
 int vwidth;
 int vplaces;
};

struct switchbox {
  char legend[128];
  int x0;
  int y0;
  int status[6];
  int width;
  int items;
};

FILE* settingsfile;

char settingsfilename[1024];

struct valuebox frequency_box;
struct valuebox centidB_box;
struct valuebox rate_box;

struct switchbox mainswitch;

int poll_sdl(void);

void show_box(valuebox);
void show_switches(switchbox);
int check_mouse_in_box(valuebox);
int check_switches(switchbox);

void update_frequency(int);
void update_level(int);
void update_srate(int);

void switch_tone_on(void);
void switch_tone_off(void);
void delay(long);
long read_settingsfile(void);
void show_status(void);
void clear_preset_switches(void);

int main(int argc,char*argv[])
{
   quit=0; reason=0;
   
   bps=16;
   
   charzero=(char)0;
   
   sprintf(appdirname,"%s",argv[0]);
   i=(int)strrchr(appdirname,'/')-(int)appdirname;
   appdirname[i]=charzero;
   /* printf("Application Dir at = %s\n\n",appdirname); */
   
   
   rates_available[0]=44100.0; rates_available[1]=48000.0;
   rates_available[2]=88200.0; rates_available[3]=96000.0;
   rind=1;
   
   read_settingsfile();
   
   f=f_preset[0]; a=a_preset[0]; r=r_preset[0]; centi_a=(int)(100.0*a);
   
   f_playing=f; a_playing=a; r_playing=r;
   
   
   tone_on=0; switch_on=0; pending=0;
   
   sprintf(frequency_box.name," Frequency = ");
   sprintf(frequency_box.units,"kHz ");
   frequency_box.value=(int)f;
   frequency_box.x0=20; frequency_box.y0=130;
   frequency_box.vwidth=5; frequency_box.vplaces=3;
   
   sprintf(centidB_box.name,"   Level   = ");
   sprintf(centidB_box.units," dB ");
   centidB_box.value=centi_a;
   centidB_box.x0=20; centidB_box.y0=100;
   centidB_box.vwidth=5; centidB_box.vplaces=2;
   
   sprintf(rate_box.name,"Sample Rate = ");
   sprintf(rate_box.units,"s/s");
   rate_box.value=(int)r;
   rate_box.x0=20;  rate_box.y0=70;
   rate_box.vwidth=5; rate_box.vplaces=0;
   
   sprintf(mainswitch.legend," ON  P1  P2  P3  P4  P5 ");
   mainswitch.x0=20; mainswitch.y0=30;
   mainswitch.items=6; 
   mainswitch.status[0]=0; mainswitch.status[1]=0;
   mainswitch.status[2]=0; mainswitch.status[3]=0;
   mainswitch.status[4]=0; mainswitch.status[5]=0;
   
   PLOT_open_window(570,200);
   
   PLOT_setRGB(80,70,50);
   PLOT_rectangle_fill(0,0,570,200);
   
   PLOT_set_font(4);
   PLOT_setRGB(200,200,200);
   PLOT_string(5,170,"ROXtone V1.02 (Jan 2013) Simple ALSA sinewave generator.");
   PLOT_move(2,165); PLOT_draw(568,165);
   show_box(frequency_box);
   show_box(centidB_box);
   show_box(rate_box);
   show_switches(mainswitch);
   show_status();

   PLOT_show();

   istop=0;
   do
   {
     reason=poll_sld();
     
     /*
     if (reason >0)
     {
       PLOT_setRGB(0,0,0);
       PLOT_rectangle_fill(400,10,300,100);
       PLOT_setRGB(255,255,255);
       PLOT_rectangle(400,10,300,100);
     }
     */
     
     if (reason==1)
     {
       /*
       sprintf(mstring,"Mouse moved to %4d %4d",mouse_x,mouse_y);
       PLOT_string(100,50,mstring);
       */
     }
     
     if(reason==2)
     {
       /*
       sprintf(mstring,"Button %2d at %4d %4d",mouse_button,mouse_x,mouse_y);
       PLOT_string(400,70,mstring);
       */
       mouse_hit=0;
       mouse_hit=check_mouse_in_box(frequency_box);
       if(mouse_hit>=0)
       {
         sprintf(mstring,"freq hit %5d",mouse_hit);
         
         update_frequency(mouse_hit);
         /*
         PLOT_string(400,50,mstring);
         */
       }
       
       mouse_hit=0;
       mouse_hit=check_mouse_in_box(centidB_box);
       if(mouse_hit>=0)
       {
         sprintf(mstring,"level hit %5d",mouse_hit);
         
         update_level(mouse_hit);
         /*
         PLOT_string(400,20,mstring);
         */
       }  
       
       
       mouse_hit=0;
       mouse_hit=check_mouse_in_box(rate_box);
       if(mouse_hit>=0)
       {
         sprintf(mstring,"rate hit ok %d",mouse_hit);
         
         update_srate(mouse_hit);
         /*
         PLOT_string(400,20,mstring);
         */
       }
       
       
       
       mouse_hit=0;
       mouse_hit=check_switches(mainswitch);
       if(mouse_hit>0)
       {
         /*
         printf("switch hit %d\n",mouse_hit);
         */
         if(mouse_hit==1)
         {
           if((tone_on==0)&&(mouse_button==1))
           {
             switch_tone_on();
           }
           else
           {
             switch_tone_off();
           }      
         }
         else
         {
           /*
           printf("Preset %d hit\n",mouse_hit-1);
           */
           f=f_preset[mouse_hit-2];
           a=a_preset[mouse_hit-2];
           r=r_preset[mouse_hit-2];
           rate_box.value=(int)r;
           centi_a=(int)(100.0*a);
           centidB_box.value=centi_a;
           frequency_box.value=(int)f;
           
           clear_preset_switches();
           
           
           
           
           mainswitch.status[mouse_hit-1]=1;
           
           
           show_box(frequency_box);
           show_box(centidB_box);
           show_box(rate_box);
           show_switches(mainswitch);
           PLOT_show();
           
         }
         
         
         
       }
     }
     
     if(reason==3)
     {
       sprintf(mstring,"Quit chosen");
       PLOT_string(360,40,mstring);
       istop=1;
     }
     
      PLOT_show();
      
      
      SDL_Delay(200);
      
   } while (istop==0);
   
   /*
   sprintf(mstring,"Loop done! Quit = %d",quit);
   
   PLOT_string(400,100,mstring);
   PLOT_string(400,200,"Plotting done! Will Quit");
   */
   
   if(tone_on>0) switch_tone_off();
   sprintf(mstring,"Quit chosen");
   PLOT_string(360,40,mstring);
   PLOT_show();
   SDL_Delay(2000);

   return 0;
}

void clear_preset_switches(void)
{
  int ics;
  ics=1;
  do
  {
    mainswitch.status[ics]=0;
    ics++;
  } while (ics<6);
}

void show_status(void)
{
  int stx0,sty0,stwidth,stheight;
  char statestring[256];
  stx0=300; sty0=30;
  stwidth=250; stheight=120;
  
  PLOT_setRGB(0,50,0);
  PLOT_rectangle_fill(stx0,sty0,stwidth,stheight);
  PLOT_setRGB(150,0,0);
  PLOT_rectangle(stx0,sty0,stwidth,stheight);
  PLOT_setRGB(255,255,90);
  if(tone_on==1)
  {
  
    sty0+=stheight-20; stx0+=10;
    PLOT_string(stx0,sty0,"Playing");
    sty0-=20;
    sprintf(statestring,"Freq  = %8.3lf kHz",f_playing*0.001);
    PLOT_string(stx0,sty0,statestring);
    sty0-=20;
    sprintf(statestring,"Level = %6.2lf db",a_playing);
    PLOT_string(stx0,sty0,statestring);
    sty0-=20;
    sprintf(statestring,"rate = %6d samp/sec",(int)r_playing);
    PLOT_string(stx0,sty0,statestring);
    sty0-=20;
    sprintf(statestring,"bits = %3d per sample",bps);
    PLOT_string(stx0,sty0,statestring);
  }
  else
  {
    sty0+=stheight-20; stx0+=10;
    PLOT_string(stx0,sty0,"Output OFF");
    
  }
  
  
  
}

void switch_tone_on(void)
{
   tone_on=1;

   mainswitch.status[0]=1;

   chld_pid=fork();
   if(0==chld_pid)
   {
     char args[10][1024];
     char *asdf[11]={
     args[0],args[1],args[2],args[3],args[4],NULL};

     sprintf(args[0],"%s/rox_siggen",appdirname);
     sprintf(args[1],"-a %6.2lf",a);
     sprintf(args[2],"-f %8.1lf",f);
     sprintf(args[3],"-r %d",(int)r);
     sprintf(args[4],"-s %d",bps);
     execv(args[0],asdf);

   }

   
   f_playing=f; a_playing=a; r_playing=r;


   clear_preset_switches();
   show_switches(mainswitch);
   show_status();

   PLOT_show();
  
}

void switch_tone_off(void)
{
  tone_on=0;
   mainswitch.status[0]=0;


   puts("killing");
   kill(chld_pid,SIGTERM);
   waitpid(chld_pid,NULL,0);
   show_switches(mainswitch);
   show_status();
   PLOT_show();
}

void update_srate(int dr)
{
  /*
  printf("Hi rate update\n");
  */
  if(mouse_button==1)
  {
    rind++;
    if (rind>3) rind=3;
  }
  else
  {
    rind--;
    if (rind<0) rind=0;
  }
  
  r=rates_available[rind];
  /*
  printf("new r = %d\n",(int)r);
  */
  rate_box.value=(int)r;
  show_box(rate_box);
  clear_preset_switches();
  show_switches(mainswitch);
  PLOT_show();
  
}


void update_level(int dn)
{
  int da;
  
  switch(dn)
  {
    case 1:
      da=1;
      break;
    case 2:
      da=10;
      break;
    case 3:
      da=100;
      break;
    case 4:
      da=1000;
      break;
  }
  
  if((mouse_button==1)||(mouse_button==4))
  {
    centi_a+=da;
    if (centi_a>0) centi_a=0;
  }
    
  if((mouse_button==3)||(mouse_button==5))
  {
    centi_a-=da;
    if(centi_a<(-9999)) centi_a=(-9999);
  }
    
   a=0.01*(float)centi_a;
   /*
   printf("a = %7.2lf\n",a);
    */
   centidB_box.value=centi_a;
   show_box(centidB_box);
   clear_preset_switches();
   show_switches(mainswitch);
   PLOT_show();
  
}

void update_frequency(int dn)
{
  int df;
  
  
  switch (dn)
  {
    case 1:
      df=1;
      break;
    case 2:
      df=10;
      break;
    case 3:
      df=100;
      break;
    case 4:
      df=1000;
      break;
    case 5:
      df=10000;
      break;
   }
      
    if((mouse_button==1)||(mouse_button==4))
    {
      f+=df;
      if (f>(float)(r)/2.0)
      {
        f=(float)(r)/2.0;
      }
    }
    
    if((mouse_button==3)||(mouse_button==5))
    {
      f-=df;
      if(f<1) f=1;
    }
   frequency_box.value=(int)f;
   show_box(frequency_box);
   clear_preset_switches();
   show_switches(mainswitch);
   PLOT_show();
}

int check_switches(struct switchbox box)
{
  int bwidth,swidth,answer,cx,cy;
  answer=-1;
  bwidth=( 1 + strlen(mainswitch.legend) ) * PLOT_cw;
  swidth=bwidth/box.items;
  /*
  printf("b %d s %d\n",bwidth,swidth);
  */
  cx=mouse_x-box.x0; cy=mouse_y-box.y0;
  
  
  if( (cx>0) && (cx<bwidth) )
  {
    if( (cy>0) && (cy<PLOT_ch) )
    {
     answer=1 + cx/swidth;
    }
  }
  return answer;
}

int check_mouse_in_box(struct valuebox box)
{
  int answer,bwidth,cx,uwidth,cy,din;
  char boxstring[128];
  answer=-1;
  
  uwidth=strlen(box.units) * PLOT_cw;
  
  cx=mouse_x-box.x0; cy=mouse_y-box.y0;
  
  
  sprintf(boxstring,"%s %05d %s",box.name,box.value,box.units);
  
  
  bwidth= (1.0+strlen(boxstring)) * PLOT_cw;
  din= ((bwidth - uwidth) - cx -4)/PLOT_cw;
  
  
  if( (cx>0) && (cx<bwidth) )
  {
    if( (cy>0) && (cy<PLOT_ch) )
    {
     answer=din;
    }
  }
  
    
  return answer;
}

void show_switches(struct switchbox box)
{
  int swidth,sheight,bx,bi;
  
  swidth=( 1 + strlen(mainswitch.legend) ) * PLOT_cw;
  swidth=swidth/box.items;
  sheight= PLOT_ch;
  /*
  printf("show switches\n");
  */
  if(tone_on>0) box.status[0]=1;
  
  bi=0;
  do
  {
    bx=box.x0+swidth*bi;
    PLOT_setRGB(100,255,100);
    PLOT_rectangle(bx,box.y0,swidth,PLOT_ch);
    PLOT_setRGB(125,125,125);
    PLOT_circle(bx+swidth/2,box.y0+PLOT_ch+10,6);
    if(box.status[bi]>0)
    {
      PLOT_setRGB(255,50,50);
      PLOT_circle_fill(bx+swidth/2,box.y0+PLOT_ch+10,6);
      PLOT_setRGB(255,250,250);
      PLOT_circle(bx+swidth/2,box.y0+PLOT_ch+10,6);
    }
    else
    {
      PLOT_setRGB(0,0,0);
      PLOT_circle_fill(bx+swidth/2,box.y0+PLOT_ch+10,6);
      PLOT_setRGB(255,250,250);
      PLOT_circle(bx+swidth/2,box.y0+PLOT_ch+10,6);
    }
    
    bi++;
  } while (bi<box.items);
  
  PLOT_setRGB(100,250,100);
  PLOT_string(box.x0+5,box.y0+4,box.legend);
  
}

void show_box(struct valuebox box)
{
  int bwidth,bheight,valint,bx,bi,uw;
  char boxstring[128];
  
  valint=(int)box.value;
  bheight= PLOT_ch;
  
  bwidth=(0.5+strlen(box.name)) * PLOT_cw;
  bx=bwidth+box.x0 + PLOT_cw;

  sprintf(boxstring,"%s %05d %s",box.name,valint,box.units);
  
  bwidth= (1.0+strlen(boxstring)) * PLOT_cw;
  
  PLOT_setRGB(50,50,50);
  
  PLOT_rectangle_fill(box.x0,box.y0,bwidth,bheight);
  PLOT_setRGB(100,250,250);
  PLOT_rectangle(box.x0,box.y0,bwidth,bheight);
  
  
  bi=0;
  
  do
  {
    PLOT_setRGB(50,50,50);
    PLOT_rectangle_fill(bx,box.y0,PLOT_cw,PLOT_ch);
  
    PLOT_setRGB(155,50,50);
    PLOT_rectangle(bx,box.y0,PLOT_cw,PLOT_ch);
    
    if( (box.vwidth-box.vplaces)==bi)
    {
      PLOT_setRGB(100,250,250);
      PLOT_circle_fill((bx + 1),(box.y0 + PLOT_ch/2 - 6),2);
      PLOT_setRGB(50,150,150);
      PLOT_move(bx,box.y0);
      PLOT_draw(bx,box.y0+PLOT_ch);
    }
    
    bi++;
    bx+=PLOT_cw;
  } while (bi<box.vwidth);
    
  PLOT_setRGB(100,250,250);
  PLOT_string(box.x0+5,box.y0+4,boxstring);
  PLOT_rectangle(box.x0,box.y0,bwidth,bheight);
  
  bx=box.x0+(strlen(boxstring)-strlen(box.units)-box.vplaces) * PLOT_cw;
  
  PLOT_setRGB(255,255,255);
}
  

int poll_sld(void)
{
  int poll_result;
  poll_result=0;
  while(SDL_PollEvent(&event))
  {
    /*
      if( event.type == SDL_MOUSEMOTION )
      {
        mouse_x=event.motion.x;
        mouse_y=event.motion.y;
        poll_result=1;
      }
    */
      if( event.type == SDL_MOUSEBUTTONDOWN )
      {
        mouse_x=event.button.x;
        mouse_y=PLOT_wheight-event.button.y;
        mouse_button=event.button.button;
        poll_result=2;
      }
      if ( event.type == SDL_QUIT )
      {
        quit=1;
        poll_result=3;
      }
  }
  return poll_result;
}

void delay(long dtime)
{
  long td0,td1,tdum,tdel;
  tdel=0;
  td0=(long)clock();
  td0=(100*td0)/CLOCKS_PER_SEC;
  do
  {
    tdum=100/22.0;
    td1=(long)clock();
    td1=(100*td1)/CLOCKS_PER_SEC;
  } while ( (td1-td0)<dtime);
  printf("Delay = %ld centisec\n",td1);
}

long read_settingsfile(void)
{
  long result,ir,iq;
  int bps_test;
  float fr,ar,rr;
  char chr;
  sprintf(settingsfilename,"%s/Settings\0",appdirname);
  /*
  printf("\nSettings file at %s\n",settingsfilename);
  */
  settingsfile=fopen(settingsfilename,"rt");
  
  ir=0;
  do
  {
    fscanf(settingsfile,"%f %f %f",&fr,&ar,&rr);
    f_preset[ir]=fr;
    a_preset[ir]=ar;
    r_preset[ir]=rr;
    ir++;
  } while (ir<5);
  
  
  if(fscanf(settingsfile,"%d",&bps_test)>0)
  {
    bps=bps_test;
    /* printf("read bps = %d\n",bps); */
  }
  /*
  else
  {
    printf("No bps line\n");
  }
  */
  
  /*
  ir=0;
  do
  {
    printf("%6.3f %5.2f %6.0f\n",f_preset[ir],a_preset[ir],r_preset[ir]);
    ir++;
  } while (ir<5);
  */
  
  
  fclose(settingsfile);
  result=1;
  return result;
}

